# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field

from . import div_animation_interpolator


class DivTransitionBase(BaseDiv):

    def __init__(
        self, *,
        duration: typing.Optional[int] = None,
        interpolator: typing.Optional[div_animation_interpolator.DivAnimationInterpolator] = None,
        start_delay: typing.Optional[int] = None,
    ):
        super().__init__(
            duration=duration,
            interpolator=interpolator,
            start_delay=start_delay,
        )

    duration: typing.Optional[int] = Field(
        description="Animation duration in milliseconds.",
    )
    interpolator: typing.Optional[div_animation_interpolator.DivAnimationInterpolator] = Field(
        description="Transition speed nature.",
    )
    start_delay: typing.Optional[int] = Field(
        description="Delay in milliseconds before animation starts.",
    )


DivTransitionBase.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field


# Variable — URL as a string.
class UrlVariable(BaseDiv):

    def __init__(
        self, *,
        name: str,
        value: str,
        type: str = "url",
    ):
        super().__init__(
            type=type,
            name=name,
            value=value,
        )

    type: str = Field(default="url")
    name: str = Field(
        min_length=1, 
        description="Variable name.",
    )
    value: str = Field(
        format="uri", 
        description="Value.",
    )


UrlVariable.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field


class DivAlignmentHorizontal(str, enum.Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing
from typing import Union

from pydivkit.core import BaseDiv, Field

from . import div_neighbour_page_size, div_page_size


DivPagerLayoutMode = Union[
    div_page_size.DivPageSize,
    div_neighbour_page_size.DivNeighbourPageSize,
]



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field

from . import div_dimension


# A point with fixed coordinates.
class DivPoint(BaseDiv):

    def __init__(
        self, *,
        x: div_dimension.DivDimension,
        y: div_dimension.DivDimension,
    ):
        super().__init__(
            x=x,
            y=y,
        )

    x: div_dimension.DivDimension = Field(
        description="`X` coordinate.",
    )
    y: div_dimension.DivDimension = Field(
        description="`Y` coordinate.",
    )


DivPoint.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field


# Solid background color.
class DivSolidBackground(BaseDiv):

    def __init__(
        self, *,
        color: str,
        type: str = "solid",
    ):
        super().__init__(
            type=type,
            color=color,
        )

    type: str = Field(default="solid")
    color: str = Field(
        format="color", 
        description="Color.",
    )


DivSolidBackground.update_forward_refs()



# flake8: noqa: F405
from typing import Dict, Any

from pydivkit.core import Field, Ref
from pydivkit.div import *
from pydivkit.div import __all__ as __div_all__


def make_card(log_id: str, *divs: Div):
    return DivData(
        log_id=log_id,
        states=[
            DivDataState(state_id=state_id, div=div)
            for state_id, div in enumerate(divs)
        ],
    )


def make_div(div: Div) -> Dict[str, Any]:
    return {
        "templates": {
            tpl.template_name: tpl.template()
            for tpl in div.related_templates()
        },
        "card": make_card("card", div).dict(),
    }


__all__ = ("Field", "Ref", "make_card", "make_div") + __div_all__



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field

from . import div_size_unit


# Stroke.
class DivStroke(BaseDiv):

    def __init__(
        self, *,
        color: str,
        unit: typing.Optional[div_size_unit.DivSizeUnit] = None,
        width: typing.Optional[int] = None,
    ):
        super().__init__(
            color=color,
            unit=unit,
            width=width,
        )

    color: str = Field(
        format="color", 
        description="Stroke color.",
    )
    unit: typing.Optional[div_size_unit.DivSizeUnit] = Field(
    )
    width: typing.Optional[int] = Field(
        description="Stroke width.",
    )


DivStroke.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field


class DivAnimationInterpolator(str, enum.Enum):
    LINEAR = "linear"
    EASE = "ease"
    EASE_IN = "ease_in"
    EASE_OUT = "ease_out"
    EASE_IN_OUT = "ease_in_out"
    SPRING = "spring"



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field


# Size with a fixed aspect ratio. It counts height from width and ignores other
# specified height values.
class DivAspect(BaseDiv):

    def __init__(
        self, *,
        ratio: float,
    ):
        super().__init__(
            ratio=ratio,
        )

    ratio: float = Field(
        description="`ratio = width / height`.",
    )


DivAspect.update_forward_refs()



from __future__ import annotations

import uuid
from dataclasses import dataclass, field
from typing import Any, Dict, Mapping, Optional


@dataclass()
class _Field:
    uid: uuid.UUID = field(default_factory=uuid.uuid4)
    name: Optional[str] = field(default=None)
    default: Optional[Any] = field(default=None)
    description: Optional[str] = field(default=None)
    constraints: Dict[str, Any] = field(default_factory=dict)
    ref_to: Optional[_Field] = field(default=None)
    is_ref: bool = field(default=False)

    @property
    def field_name(self) -> str:
        if not self.name:
            raise ValueError("Field 'name' not initialized")
        return self.name

    def apply_constraints(self, constraints: Mapping[str, Any]) -> None:
        for c_key, c_value in constraints.items():
            if self.constraints.get(c_key) and self.constraints[c_key] != c_value:
                raise ValueError(
                    f"Incompatible constraints: {c_key} = "
                    f"{self.constraints[c_key]} and {c_key} = {c_value}",
                )
            self.constraints[c_key] = c_value


def _make_constraints(**kwargs: Any) -> Dict[str, Any]:
    return {k: v for k, v in kwargs.items() if v is not None}


def Field(
    name: Optional[str] = None,
    description: Optional[str] = None,
    default: Optional[Any] = None,
    format: Optional[str] = None,
    gt: Optional[float] = None,
    ge: Optional[float] = None,
    lt: Optional[float] = None,
    le: Optional[float] = None,
    multiple_of: Optional[float] = None,
    min_items: Optional[int] = None,
    max_items: Optional[int] = None,
    unique_items: Optional[bool] = None,
    min_length: Optional[int] = None,
    max_length: Optional[int] = None,
    regex: Optional[str] = None,
) -> Any:
    return _Field(
        name=name,
        description=description,
        default=default,
        constraints=_make_constraints(
            **{
                "format": format,
                "exclusiveMinimum": gt,
                "minimum": ge,
                "exclusiveMaximum": lt,
                "maximum": le,
                "multipleOf": multiple_of,
                "minItems": min_items,
                "maxItems": max_items,
                "uniqueItems": unique_items,
                "minLength": min_length,
                "maxLength": max_length,
                "regex": regex,
            },
        ),
    )


def Ref(field: Any) -> Any:
    if not isinstance(field, _Field):
        raise ValueError(
            f"Value {field} has wrong type. Expected type is Field.",
        )
    field.is_ref = True
    return _Field(ref_to=field)



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field

from . import div_radial_gradient_center, div_radial_gradient_radius


# Radial gradient.
class DivRadialGradient(BaseDiv):

    def __init__(
        self, *,
        colors: typing.List[str],
        type: str = "radial_gradient",
        center_x: typing.Optional[div_radial_gradient_center.DivRadialGradientCenter] = None,
        center_y: typing.Optional[div_radial_gradient_center.DivRadialGradientCenter] = None,
        radius: typing.Optional[div_radial_gradient_radius.DivRadialGradientRadius] = None,
    ):
        super().__init__(
            type=type,
            center_x=center_x,
            center_y=center_y,
            colors=colors,
            radius=radius,
        )

    type: str = Field(default="radial_gradient")
    center_x: typing.Optional[div_radial_gradient_center.DivRadialGradientCenter] = Field(
        description=(
            "Shift of the central point of the gradient relative to the "
            "left edge along the Xaxis."
        ),
    )
    center_y: typing.Optional[div_radial_gradient_center.DivRadialGradientCenter] = Field(
        description=(
            "Shift of the central point of the gradient relative to the "
            "upper edge along the Yaxis."
        ),
    )
    colors: typing.List[str] = Field(
        min_items=2, 
        description=(
            "Colors. Gradient points will be located at an equal "
            "distance from each other."
        ),
    )
    radius: typing.Optional[div_radial_gradient_radius.DivRadialGradientRadius] = Field(
        description="Radius of the gradient transition.",
    )


DivRadialGradient.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field


# Relative central point of the gradient.
class DivRadialGradientRelativeCenter(BaseDiv):

    def __init__(
        self, *,
        value: float,
        type: str = "relative",
    ):
        super().__init__(
            type=type,
            value=value,
        )

    type: str = Field(default="relative")
    value: float = Field(
        description=(
            "Shift value of the central relative point of the gradient "
            "in the range `0..1`."
        ),
    )


DivRadialGradientRelativeCenter.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field


# Sets the margins, without taking the properties of the screen.
class DivAbsoluteEdgeInsets(BaseDiv):

    def __init__(
        self, *,
        bottom: typing.Optional[int] = None,
        left: typing.Optional[int] = None,
        right: typing.Optional[int] = None,
        top: typing.Optional[int] = None,
    ):
        super().__init__(
            bottom=bottom,
            left=left,
            right=right,
            top=top,
        )

    bottom: typing.Optional[int] = Field(
        description="Bottom margin.",
    )
    left: typing.Optional[int] = Field(
        description="Left margin.",
    )
    right: typing.Optional[int] = Field(
        description="Right margin.",
    )
    top: typing.Optional[int] = Field(
        description="Top margin.",
    )


DivAbsoluteEdgeInsets.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field

from . import div_download_callbacks


# It defines an action when clicking on an element.
class DivAction(BaseDiv):

    def __init__(
        self, *,
        log_id: str,
        download_callbacks: typing.Optional[div_download_callbacks.DivDownloadCallbacks] = None,
        log_url: typing.Optional[str] = None,
        menu_items: typing.Optional[typing.List[DivActionMenuItem]] = None,
        payload: typing.Optional[typing.Dict[str, typing.Any]] = None,
        referer: typing.Optional[str] = None,
        target: typing.Optional[DivActionTarget] = None,
        url: typing.Optional[str] = None,
    ):
        super().__init__(
            download_callbacks=download_callbacks,
            log_id=log_id,
            log_url=log_url,
            menu_items=menu_items,
            payload=payload,
            referer=referer,
            target=target,
            url=url,
        )

    download_callbacks: typing.Optional[div_download_callbacks.DivDownloadCallbacks] = Field(
        description=(
            "Callbacks that are called after "
            "[dataloading](../../interaction.dita#loading-data)."
        ),
    )
    log_id: str = Field(
        min_length=1, 
        description="Logging ID.",
    )
    log_url: typing.Optional[str] = Field(
        format="uri", 
        description="URL for logging.",
    )
    menu_items: typing.Optional[typing.List[DivActionMenuItem]] = Field(
        min_items=1, 
        description="Context menu.",
    )
    payload: typing.Optional[typing.Dict[str, typing.Any]] = Field(
        description="Additional parameters, passed to the host application.",
    )
    referer: typing.Optional[str] = Field(
        format="uri", 
        description="Referer URL for logging.",
    )
    target: typing.Optional[DivActionTarget] = Field(
        description="The tab in which the URL must be opened.",
    )
    url: typing.Optional[str] = Field(
        format="uri", 
        description=(
            "URL. Possible values: `url` or `div-action://`. To learn "
            "more, see [Interactionwith "
            "elements](../../interaction.dita)."
        ),
    )


class DivActionTarget(str, enum.Enum):
    SELF = "_self"
    BLANK = "_blank"


class DivActionMenuItem(BaseDiv):

    def __init__(
        self, *,
        text: str,
        action: typing.Optional[DivAction] = None,
        actions: typing.Optional[typing.List[DivAction]] = None,
    ):
        super().__init__(
            action=action,
            actions=actions,
            text=text,
        )

    action: typing.Optional[DivAction] = Field(
        description=(
            "One action when clicking on a menu item. Not used if the "
            "`actions` parameter isset."
        ),
    )
    actions: typing.Optional[typing.List[DivAction]] = Field(
        min_items=1, 
        description="Multiple actions when clicking on a menu item.",
    )
    text: str = Field(
        min_length=1, 
        description="Menu item title.",
    )


DivActionMenuItem.update_forward_refs()


DivAction.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field


# Linear gradient.
class DivLinearGradient(BaseDiv):

    def __init__(
        self, *,
        colors: typing.List[str],
        type: str = "gradient",
        angle: typing.Optional[int] = None,
    ):
        super().__init__(
            type=type,
            angle=angle,
            colors=colors,
        )

    type: str = Field(default="gradient")
    angle: typing.Optional[int] = Field(
        description="Angle of gradient direction.",
    )
    colors: typing.List[str] = Field(
        min_items=2, 
        description=(
            "Colors. Gradient points will be located at an equal "
            "distance from each other."
        ),
    )


DivLinearGradient.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field


class DivImageScale(str, enum.Enum):
    FILL = "fill"
    NO_SCALE = "no_scale"
    FIT = "fit"



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field


# The size of an element adjusts to its contents.
class DivWrapContentSize(BaseDiv):

    def __init__(
        self, *,
        type: str = "wrap_content",
        constrained: typing.Optional[bool] = None,
    ):
        super().__init__(
            type=type,
            constrained=constrained,
        )

    type: str = Field(default="wrap_content")
    constrained: typing.Optional[bool] = Field(
        description=(
            "The final size mustn\'t exceed the parent one. On iOS and "
            "in a default browser`false`. On Android always `true`."
        ),
    )


DivWrapContentSize.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing
from typing import Union

from pydivkit.core import BaseDiv, Field

from . import (
    div_appearance_set_transition, div_fade_transition, div_scale_transition,
    div_slide_transition,
)


DivAppearanceTransition = Union[
    div_appearance_set_transition.DivAppearanceSetTransition,
    div_fade_transition.DivFadeTransition,
    div_scale_transition.DivScaleTransition,
    div_slide_transition.DivSlideTransition,
]



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field

from . import div_fixed_size


# A rectangle with rounded corners.
class DivRoundedRectangleShape(BaseDiv):

    def __init__(
        self, *,
        type: str = "rounded_rectangle",
        corner_radius: typing.Optional[div_fixed_size.DivFixedSize] = None,
        item_height: typing.Optional[div_fixed_size.DivFixedSize] = None,
        item_width: typing.Optional[div_fixed_size.DivFixedSize] = None,
    ):
        super().__init__(
            type=type,
            corner_radius=corner_radius,
            item_height=item_height,
            item_width=item_width,
        )

    type: str = Field(default="rounded_rectangle")
    corner_radius: typing.Optional[div_fixed_size.DivFixedSize] = Field(
        description="Corner rounding radius.",
    )
    item_height: typing.Optional[div_fixed_size.DivFixedSize] = Field(
        description="Height.",
    )
    item_width: typing.Optional[div_fixed_size.DivFixedSize] = Field(
        description="Width.",
    )


DivRoundedRectangleShape.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field


class DivFontWeight(str, enum.Enum):
    LIGHT = "light"
    MEDIUM = "medium"
    REGULAR = "regular"
    BOLD = "bold"



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field

from . import div_size_unit


# Value of the offset of the coordinates of the axis of rotation.
class DivPivotFixed(BaseDiv):

    def __init__(
        self, *,
        type: str = "pivot-fixed",
        unit: typing.Optional[div_size_unit.DivSizeUnit] = None,
        value: typing.Optional[int] = None,
    ):
        super().__init__(
            type=type,
            unit=unit,
            value=value,
        )

    type: str = Field(default="pivot-fixed")
    unit: typing.Optional[div_size_unit.DivSizeUnit] = Field(
        description=(
            "Unit of size measurement. To learn more about units of size "
            "measurement, see[Layout inside the "
            "card](../../layout.dita)."
        ),
    )
    value: typing.Optional[int] = Field(
        description="Offset.",
    )


DivPivotFixed.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field

from . import div_fixed_size


# A circle.
class DivCircleShape(BaseDiv):

    def __init__(
        self, *,
        type: str = "circle",
        radius: typing.Optional[div_fixed_size.DivFixedSize] = None,
    ):
        super().__init__(
            type=type,
            radius=radius,
        )

    type: str = Field(default="circle")
    radius: typing.Optional[div_fixed_size.DivFixedSize] = Field(
        description="Radius.",
    )


DivCircleShape.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field


# Location of the coordinate of the axis of rotation as a percentage relative to
# the element.
class DivPivotPercentage(BaseDiv):

    def __init__(
        self, *,
        value: float,
        type: str = "pivot-percentage",
    ):
        super().__init__(
            type=type,
            value=value,
        )

    type: str = Field(default="pivot-percentage")
    value: float = Field(
        description="Location of the element.",
    )


DivPivotPercentage.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field


# Fixed number of repetitions.
class DivFixedCount(BaseDiv):

    def __init__(
        self, *,
        value: int,
        type: str = "fixed",
    ):
        super().__init__(
            type=type,
            value=value,
        )

    type: str = Field(default="fixed")
    value: int = Field(
        description="Number of repetitions.",
    )


DivFixedCount.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field

from . import div_size_unit


# Element dimension value.
class DivDimension(BaseDiv):

    def __init__(
        self, *,
        value: float,
        unit: typing.Optional[div_size_unit.DivSizeUnit] = None,
    ):
        super().__init__(
            unit=unit,
            value=value,
        )

    unit: typing.Optional[div_size_unit.DivSizeUnit] = Field(
    )
    value: float = Field(
        description="Value.",
    )


DivDimension.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field

from . import div_alignment_horizontal, div_alignment_vertical, div_image_scale


# Background image.
class DivImageBackground(BaseDiv):

    def __init__(
        self, *,
        image_url: str,
        type: str = "image",
        alpha: typing.Optional[float] = None,
        content_alignment_horizontal: typing.Optional[div_alignment_horizontal.DivAlignmentHorizontal] = None,
        content_alignment_vertical: typing.Optional[div_alignment_vertical.DivAlignmentVertical] = None,
        preload_required: typing.Optional[bool] = None,
        scale: typing.Optional[div_image_scale.DivImageScale] = None,
    ):
        super().__init__(
            type=type,
            alpha=alpha,
            content_alignment_horizontal=content_alignment_horizontal,
            content_alignment_vertical=content_alignment_vertical,
            image_url=image_url,
            preload_required=preload_required,
            scale=scale,
        )

    type: str = Field(default="image")
    alpha: typing.Optional[float] = Field(
        description="Image transparency.",
    )
    content_alignment_horizontal: typing.Optional[div_alignment_horizontal.DivAlignmentHorizontal] = Field(
        description="Horizontal image alignment.",
    )
    content_alignment_vertical: typing.Optional[div_alignment_vertical.DivAlignmentVertical] = Field(
        description="Vertical image alignment.",
    )
    image_url: str = Field(
        format="uri", 
        description="Image URL.",
    )
    preload_required: typing.Optional[bool] = Field(
        description="Background image must be loaded before the display.",
    )
    scale: typing.Optional[div_image_scale.DivImageScale] = Field(
        description="Image scaling.",
    )


DivImageBackground.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field

from . import div_absolute_edge_insets


# Image in nine patch format.
# https://developer.android.com/studio/write/draw9patch.
class DivNinePatchBackground(BaseDiv):

    def __init__(
        self, *,
        image_url: str,
        insets: div_absolute_edge_insets.DivAbsoluteEdgeInsets,
        type: str = "nine_patch_image",
    ):
        super().__init__(
            type=type,
            image_url=image_url,
            insets=insets,
        )

    type: str = Field(default="nine_patch_image")
    image_url: str = Field(
        format="uri", 
        description="Image URL.",
    )
    insets: div_absolute_edge_insets.DivAbsoluteEdgeInsets = Field(
        description=(
            "Margins that break images into parts, according to the "
            "rule, are similar "
            "toborder-image-slice.(https://developer.mozilla.org/en-US/d"
            "ocs/Web/CSS/border-image-slice)"
        ),
    )


DivNinePatchBackground.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field

from . import div, div_transition_selector, div_trigger, div_variable


# Root structure.
class DivData(BaseDiv):

    def __init__(
        self, *,
        log_id: str,
        states: typing.List[DivDataState],
        transition_animation_selector: typing.Optional[div_transition_selector.DivTransitionSelector] = None,
        variable_triggers: typing.Optional[typing.List[div_trigger.DivTrigger]] = None,
        variables: typing.Optional[typing.List[div_variable.DivVariable]] = None,
    ):
        super().__init__(
            log_id=log_id,
            states=states,
            transition_animation_selector=transition_animation_selector,
            variable_triggers=variable_triggers,
            variables=variables,
        )

    log_id: str = Field(
        min_length=1, 
        description="Logging ID.",
    )
    states: typing.List[DivDataState] = Field(
        min_items=1, 
        description=(
            "A set of visual element states. Each element can have a few "
            "states with adifferent layout. The states are displayed "
            "strictly one by one and switched "
            "using[action](div-action.md)."
        ),
    )
    transition_animation_selector: typing.Optional[div_transition_selector.DivTransitionSelector] = Field(
        description="Events that trigger transition animations. @deprecated",
    )
    variable_triggers: typing.Optional[typing.List[div_trigger.DivTrigger]] = Field(
        min_items=1, 
        description="Triggers for changing variables.",
    )
    variables: typing.Optional[typing.List[div_variable.DivVariable]] = Field(
        min_items=1, 
        description="Declaration of variables that can be used in an element.",
    )


class DivDataState(BaseDiv):

    def __init__(
        self, *,
        div: div.Div,
        state_id: int,
    ):
        super().__init__(
            div=div,
            state_id=state_id,
        )

    div: div.Div = Field(
        description="Contents.",
    )
    state_id: int = Field(
        description="State ID.",
    )


DivDataState.update_forward_refs()


DivData.update_forward_refs()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pydivkit', 'pydivkit.core', 'pydivkit.core.types', 'pydivkit.div']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'pydivkit',
    'version': '13.0.0b0',
    'description': 'DivKit python library',
    'long_description': 'PyDIVKit examples\n=================\n\nThis library is for working with DivKit with python.\n\nFeatures:\n* Declarative and imperative DivKit blocks definition\n* Native Type-hints support\n* Complete object-oriented API\n* IDE type checks and suggestions\n\nObject construction\n-------------------\n\nThe main idea is a provides ability to construct blocks by python objects.\n\n```python\nimport json\nimport pydivkit as dk\n\ncontainer = dk.DivContainer(\n    items=[\n        dk.DivGallery(\n            items=[\n                dk.DivText(text="Hello from pydivkit")\n            ]\n        )\n    ]\n)\n\nprint(json.dumps(container.dict(), indent=1))\n# {\n#  "type": "container",\n#  "items": [\n#   {\n#    "type": "gallery",\n#    "items": [\n#     {\n#      "type": "text",\n#      "text": "Hello from pydivkit"\n#     }\n#    ]\n#   }\n#  ]\n# }\n```\n\nSlider example\n--------------\n\nFollowing code is a rewritten slider example using pydivkit.\n\n```python\nimport pydivkit as dk\n\n\nslider = dk.DivData(\n    log_id="sample_card",\n    states=[\n        dk.DivDataState(\n            state_id=0,\n            div=dk.DivSlider(\n                width=dk.DivMatchParentSize(),\n                max_value=10,\n                min_value=1,\n                thumb_style=dk.DivShapeDrawable(\n                    color="#00b300",\n                    stroke=dk.DivStroke(\n                        color="#ffffff",\n                        width=3,\n                    ),\n                    shape=dk.DivRoundedRectangleShape(\n                        item_width=dk.DivFixedSize(value=32),\n                        item_height=dk.DivFixedSize(value=32),\n                        corner_radius=dk.DivFixedSize(value=100)\n                    ),\n                ),\n                track_active_style=dk.DivShapeDrawable(\n                    color="#00b300",\n                    shape=dk.DivRoundedRectangleShape(\n                        item_height=dk.DivFixedSize(value=6)\n                    )\n                ),\n                track_inactive_style=dk.DivShapeDrawable(\n                    color="#20000000",\n                    shape=dk.DivRoundedRectangleShape(\n                        item_height=dk.DivFixedSize(value=6)\n                    )\n                )\n            )\n        )\n    ]\n)\n\n```\n\nThis example might be serialised like this:\n\n```python\nimport json\n\nprint(json.dumps(slider.dict(), indent=1))\n# {\n#  "log_id": "sample_card",\n#  "states": [\n#   {\n#    "div": {\n#     "type": "slider",\n#     "max_value": 10,\n#     "min_value": 1,\n#     "thumb_style": {\n#      "type": "shape_drawable",\n#      "color": "#00b300",\n#      "shape": {\n#       "type": "rounded_rectangle",\n#       "corner_radius": {\n#        "type": "fixed",\n#        "value": 100\n#       },\n#       "item_height": {\n#        "type": "fixed",\n#        "value": 32\n#       },\n#       "item_width": {\n#        "type": "fixed",\n#        "value": 32\n#       }\n#      },\n#      "stroke": {\n#       "color": "#ffffff",\n#       "width": 3\n#      }\n#     },\n#     "track_active_style": {\n#      "type": "shape_drawable",\n#      "color": "#00b300",\n#      "shape": {\n#       "type": "rounded_rectangle",\n#       "item_height": {\n#        "type": "fixed",\n#        "value": 6\n#       }\n#      }\n#     },\n#     "track_inactive_style": {\n#      "type": "shape_drawable",\n#      "color": "#20000000",\n#      "shape": {\n#       "type": "rounded_rectangle",\n#       "item_height": {\n#        "type": "fixed",\n#        "value": 6\n#       }\n#      }\n#     },\n#     "width": {\n#      "type": "match_parent"\n#     }\n#    },\n#    "state_id": 0\n#   }\n#  ]\n# }\n```\n\nTemplating and DRY\n------------------\n\nOf course building blocks from your code every time manually it\'s\na very boring. So the first idea it\'s a move initialization of the \nDivKit objects to the functions.\n\n```python\n# Naive DRY example which strictly non-recommended\nimport pydivkit as dk\n\n\ndef get_size(value: int = 32) -> dk.DivFixedSize:\n    return dk.DivFixedSize(value=value)\n\n\ndef get_shape() -> dk.DivShape:\n    return dk.DivShape(\n        item_width=get_size(),\n        item_height=get_size(),\n        corner_radius=get_size(100)\n    )\n\n    \nslider_shape = get_shape()\n\nslider = dk.DivData(\n    log_id="sample_card",\n    states=[\n        dk.DivDataState(\n            # other arguments\n            div=dk.DivSlider(\n                thumb_style=dk.DivShapeDrawable(\n                    shape=slider_shape,\n                    # other arguments\n                ),\n                # other arguments\n            )\n        )\n    ]\n)\n```\n\nLooks a little better, but this approach doesn\'t scale well. To simplify layout \nand save traffic, DivKit has templates. This is a way to layout similar elements\nwithout having to declare the complete json, but just declare a template and use\nthis many times in similar items.\n\n**PyDivKit supports defining templates through the inheritance.**\n\nLet\'s define an example card.\n\n```python\nimport json\n\nimport pydivkit as dk\n\n\nclass CategoriesItem(dk.DivContainer):\n    """\n    Class inherited from dk.DivContainer will have a template\n    """\n\n    # Special object for mark this fields a DivKit field in template\n    icon_url: str = dk.Field()\n    text: str = dk.Field()\n\n    # Set defaults layout for in the template\n    width = dk.DivWrapContentSize()\n    background = [dk.DivSolidBackground(color="#f0f0f0")]\n    content_alignment_vertical = dk.DivAlignmentVertical.CENTER\n    orientation = dk.DivContainerOrientation.HORIZONTAL\n    paddings = dk.DivEdgeInsets(left=12, right=12, top=10, bottom=10)\n    border = dk.DivBorder(corner_radius=12)\n    items = [\n        dk.DivImage(\n            width=dk.DivFixedSize(value=20),\n            height=dk.DivFixedSize(value=20),\n            margins=dk.DivEdgeInsets(right=6),\n\n            # Special object Ref it\'s a reference for Field property\n            image_url=dk.Ref(icon_url),\n        ),\n        dk.DivText(\n            width=dk.DivWrapContentSize(),\n            max_lines=1,\n\n            # Special object Ref it\'s a reference for Field property\n            text=dk.Ref(text),\n        ),\n    ]\n\n\nBASE_URL = "https://leonardo.edadeal.io/dyn/re/segments/level1/96"\n\n\n# So after class definition you might use all the `Field` marked property\n# names as an argument.\n\ngallery = dk.DivGallery(\n    items=[\n        CategoriesItem(\n            text="Food", icon_url=f"{BASE_URL}/food.png",\n        ),\n        CategoriesItem(\n            text="Alcohol", icon_url=f"{BASE_URL}/alcohol.png",\n        ),\n        CategoriesItem(\n            text="Household", icon_url=f"{BASE_URL}/household.png",\n        ),\n    ]\n)\n\nprint(json.dumps(dk.make_div(gallery), indent=1, ensure_ascii=False))\n# {\n#  "templates": {\n#   "__main__.CategoriesItem": {\n#    "type": "container",\n#    "background": [\n#     {\n#      "type": "solid",\n#      "color": "#f0f0f0"\n#     }\n#    ],\n#    "border": {\n#     "corner_radius": 12\n#    },\n#    "content_alignment_vertical": "center",\n#    "items": [\n#     {\n#      "type": "image",\n#      "height": {\n#       "type": "fixed",\n#       "value": 20\n#      },\n#      "$image_url": "icon_url",\n#      "margins": {\n#       "right": 6\n#      },\n#      "width": {\n#       "type": "fixed",\n#       "value": 20\n#      }\n#     },\n#     {\n#      "type": "text",\n#      "max_lines": 1,\n#      "$text": "text",\n#      "width": {\n#       "type": "wrap_content"\n#      }\n#     }\n#    ],\n#    "orientation": "horizontal",\n#    "paddings": {\n#     "bottom": 10,\n#     "left": 12,\n#     "right": 12,\n#     "top": 10\n#    },\n#    "width": {\n#     "type": "wrap_content"\n#    }\n#   }\n#  },\n#  "card": {\n#   "log_id": "card",\n#   "states": [\n#    {\n#     "div": {\n#      "type": "gallery",\n#      "items": [\n#       {\n#        "type": "__main__.CategoriesItem",\n#        "icon_url": "https://leonardo.edadeal.io/dyn/re/segments/level1/96/food.png",\n#        "text": "Food"\n#       },\n#       {\n#        "type": "__main__.CategoriesItem",\n#        "icon_url": "https://leonardo.edadeal.io/dyn/re/segments/level1/96/alcohol.png",\n#        "text": "Alcohol"\n#       },\n#       {\n#        "type": "__main__.CategoriesItem",\n#        "icon_url": "https://leonardo.edadeal.io/dyn/re/segments/level1/96/household.png",\n#        "text": "Household"\n#       }\n#      ]\n#     },\n#     "state_id": 0\n#    }\n#   ]\n#  }\n# }\n```\n\nTemplate names\n--------------\n\nBy default, templates are collecting by the metaclass into shared storage when \nthe class is declaring at import time, and have the \nformat `{module_name}.{class_name}`.\n\nThe following example, which I am sure will not occur in the wild, shows a \nwarning if suddenly the names of the classes, and hence the templates, conflict.\n\n```python\nimport pydivkit as dk\n\n\nclass MyTemplate(dk.DivContainer):\n    width = dk.DivWrapContentSize()\n\n\nclass MyTemplate(dk.DivContainer):\n    pass\n\n# RuntimeWarning: Template \'test.MyTemplate\' already defined in \n# <class \'test.MyTemplate\'> and will be replaced to <class \'test.MyTemplate\'>\n```\n\nAlso, if you do not want to show the structure of your project to the outside, \nor for some reason you need to make the example above clean, you can rename the\ntemplate by declaring a special attribute `__template_name__`\n\n```python\nimport pydivkit as dk\n\n\nclass MyTemplate(dk.DivContainer):\n    width = dk.DivWrapContentSize()\n\nprint(MyTemplate.template_name)\n# >>> test.MyTemplate\n\nclass MyTemplate(dk.DivContainer):\n    __template_name__ = "MyTemplate2"\n\nprint(MyTemplate.template_name)\n# >>> MyTemplate2\n```\n',
    'author': 'Vladislav Bakaev',
    'author_email': 'bakaev-vlad@yandex-team.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/divkit/divkit/tree/main/json-builder/python',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Field

from . import div_action, div_background, div_border


# Element behavior when focusing or losing focus.
class DivFocus(BaseDiv):

    def __init__(
        self, *,
        background: typing.Optional[typing.List[div_background.DivBackground]] = None,
        border: typing.Optional[div_border.DivBorder] = None,
        next_focus_ids: typing.Optional[DivFocusNextFocusIds] = None,
        on_blur: typing.Optional[typing.List[div_action.DivAction]] = None,
        on_focus: typing.Optional[typing.List[div_action.DivAction]] = None,
    ):
        super().__init__(
            background=background,
            border=border,
            next_focus_ids=next_focus_ids,
            on_blur=on_blur,
            on_focus=on_focus,
        )

    background: typing.Optional[typing.List[div_background.DivBackground]] = Field(
        min_items=1, 
        description=(
            "Background of an element when it is in focus. It can "
            "contain multiple layers."
        ),
    )
    border: typing.Optional[div_border.DivBorder] = Field(
        description="Border of an element when it is in focus",
    )
    next_focus_ids: typing.Optional[DivFocusNextFocusIds] = Field(
        description="IDs of elements that will be next to get focus.",
    )
    on_blur: typing.Optional[typing.List[div_action.DivAction]] = Field(
        min_items=1, 
        description="Actions when an element loses focus.",
    )
    on_focus: typing.Optional[typing.List[div_action.DivAction]] = Field(
        min_items=1, 
        description="Actions when an element gets focus.",
    )


# IDs of elements that will be next to get focus.
class DivFocusNextFocusIds(BaseDiv):

    def __init__(
        self, *,
        down: typing.Optional[str] = None,
        forward: typing.Optional[str] = None,
        left: typing.Optional[str] = None,
        right: typing.Optional[str] = None,
        up: typing.Optional[str] = None,
    ):
        super().__init__(
            down=down,
            forward=forward,
            left=left,
            right=right,
            up=up,
        )

    down: typing.Optional[str] = Field(
        min_length=1,
    )
    forward: typing.Optional[str] = Field(
        min_length=1,
    )
    left: typing.Optional[str] = Field(
        min_length=1,
    )
    right: typing.Optional[str] = Field(
        min_length=1,
    )
    up: typing.Optional[str] = Field(
        min_length=1,
    )


DivFocusNextFocusIds.update_forward_refs()


DivFocus.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing
from typing import Union

from pydivkit.core import BaseDiv, Field

from . import div_circle_shape, div_rounded_rectangle_shape


DivShape = Union[
    div_rounded_rectangle_shape.DivRoundedRectangleShape,
    div_circle_shape.DivCircleShape,
]



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing
from typing import Union

from pydivkit.core import BaseDiv, Field

from . import (
    div_image_background, div_linear_gradient, div_nine_patch_background,
    div_radial_gradient, div_solid_background,
)


DivBackground = Union[
    div_linear_gradient.DivLinearGradient,
    div_radial_gradient.DivRadialGradient,
    div_image_background.DivImageBackground,
    div_solid_background.DivSolidBackground,
    div_nine_patch_background.DivNinePatchBackground,
]
